@@ -90,6 +90,8 @@ protected:
 		}
 		guint key = gdk_keyval_to_lower(event->keyval);
 		guint mods = event->state & gtk_accelerator_get_default_mod_mask();
+		if (key == event->keyval)
+			mods &= ~GDK_Shift_L & ~GDK_Shift_R;
 
 		editing_done();
 		remove_widget();
@@ -219,7 +221,6 @@ Actions::Actions() :
 	tm = Store::create(cols, this);
 
 	tv.get_selection()->set_mode(Gtk::SELECTION_MULTIPLE);
-	tv.set_row_separator_func(sigc::mem_fun(*this, &Actions::on_row_separator));
 
 	int n;
 	n = tv.append_column(_("Stroke"), cols.stroke);
@@ -236,8 +237,8 @@ Actions::Actions() :
 	name_renderer->signal_editing_canceled().connect(sigc::mem_fun(*this, &Actions::on_something_editing_canceled));
 	Gtk::TreeView::Column *col_name = tv.get_column(n-1);
 	col_name->set_sort_column(cols.name);
-//	col_name->add_attribute(name_renderer->property_text(), cols.name);
 	col_name->set_cell_data_func(*name_renderer, sigc::mem_fun(*this, &Actions::on_cell_data_name));
+	col_name->set_resizable();
 
 	type_store = Gtk::ListStore::create(type);
 	for (TypeInfo *i = all_types; i->name; i++)
@@ -262,6 +263,7 @@ Actions::Actions() :
 	Gtk::TreeView::Column *col_arg = tv.get_column(n-1);
 	col_arg->add_attribute(arg_renderer->property_text(), cols.arg);
 	col_arg->set_cell_data_func(*arg_renderer, sigc::mem_fun(*this, &Actions::on_cell_data_arg));
+	col_arg->set_resizable();
 	arg_renderer->property_editable() = true;
 	arg_renderer->signal_key_edited().connect(sigc::mem_fun(*this, &Actions::on_accel_edited));
 	arg_renderer->signal_combo_edited().connect(sigc::mem_fun(*this, &Actions::on_combo_edited));
@@ -280,13 +282,16 @@ Actions::Actions() :
 	apps_model = AppsStore::create(ca, this);
 
 	load_app_list(apps_model->children(), actions.get_root());
+	update_counts();
 
 	apps_view->append_column_editable(_("Application"), ca.app);
+	apps_view->get_column(0)->set_expand(true);
 	apps_view->get_column(0)->set_cell_data_func(
 			*apps_view->get_column_cell_renderer(0), sigc::mem_fun(*this, &Actions::on_cell_data_apps));
 	Gtk::CellRendererText *app_name_renderer =
 		dynamic_cast<Gtk::CellRendererText *>(apps_view->get_column_cell_renderer(0));
 	app_name_renderer->signal_edited().connect(sigc::mem_fun(*this, &Actions::on_group_name_edited));
+	apps_view->append_column(_("Actions"), ca.count);
 
 	apps_view->set_model(apps_model);
 	apps_view->enable_model_drag_dest();
@@ -295,7 +300,7 @@ Actions::Actions() :
 
 void Actions::load_app_list(const Gtk::TreeNodeChildren &ch, ActionListDiff *actions) {
 	Gtk::TreeRow row = *(apps_model->append(ch));
-	row[ca.app] = actions->name;
+	row[ca.app] = app_name_hr(actions->name);
 	row[ca.actions] = actions;
 	for (ActionListDiff::iterator i = actions->begin(); i != actions->end(); i++)
 		load_app_list(row.children(), &(*i));
@@ -332,10 +337,6 @@ void Actions::on_cell_data_arg(Gtk::CellRenderer* cell, const Gtk::TreeModel::it
 	renderer->mode = all_types[from_name(str)].mode;
 }
 
-bool Actions::on_row_separator(const Glib::RefPtr<Gtk::TreeModel> &model, const Gtk::TreeModel::iterator &iter) {
-	return false; //(*iter)[cols.separator];
-}
-
 int Actions::compare_ids(const Gtk::TreeModel::iterator &a, const Gtk::TreeModel::iterator &b) {
 	Unique *x = (*a)[cols.id];
 	Unique *y = (*b)[cols.id];
@@ -512,7 +513,7 @@ void Actions::on_type_edited(const Glib::ustring &path, const Glib::ustring &new
 			row[cols.cmd_save] = (Glib::ustring)row[cols.arg];
 		}
 		if (new_type == KEY) {
-			new_action = SendKey::create(0, (Gdk::ModifierType)0, 0);
+			new_action = SendKey::create(0, (Gdk::ModifierType)0);
 			edit = true;
 		}
 		if (new_type == TEXT) {
@@ -574,6 +575,7 @@ void Actions::on_button_delete() {
 	}
 	update_action_list();
 	update_actions();
+	update_counts();
 }
 
 void Actions::on_cell_data_apps(Gtk::CellRenderer* cell, const Gtk::TreeModel::iterator& iter) {
@@ -604,7 +606,7 @@ void Actions::on_add_app() {
 		apps_model->children().begin()->children() :
 		apps_view->get_selection()->get_selected()->children();
 	Gtk::TreeRow row = *(apps_model->append(ch));
-	row[ca.app] = name;
+	row[ca.app] = app_name_hr(name);
 	row[ca.actions] = child;
 	actions.apps[name] = child;
 	Gtk::TreePath path = apps_model->get_path(row);
@@ -694,6 +696,15 @@ void Actions::on_apps_selection_changed() {
 	}
 }
 
+bool Actions::count_app_actions(const Gtk::TreeIter &i) {
+	(*i)[ca.count] = ((ActionListDiff*)(*i)[ca.actions])->count_actions();
+	return false;
+}
+
+void Actions::update_counts() {
+	apps_model->foreach_iter(sigc::mem_fun(*this, &Actions::count_app_actions));
+}
+
 void Actions::update_action_list() {
 	boost::shared_ptr<std::set<Unique *> > ids = action_list->get_ids(check_show_deleted->get_active());
 	const Gtk::TreeNodeChildren &ch = tm->children();
@@ -878,6 +889,7 @@ void Actions::on_button_new() {
 	update_row(row);
 	focus(id, 1, true);
 	update_actions();
+	update_counts();
 }
 
 bool Actions::do_focus(Unique *id, Gtk::TreeViewColumn *col, bool edit) {
@@ -920,7 +932,7 @@ void Actions::on_accel_edited(const Glib::ustring& path_string, guint accel_key,
 	Gtk::TreeRow row(*tm->get_iter(path_string));
 	Type type = from_name(row[cols.type]);
 	if (type == KEY) {
-		RSendKey send_key = SendKey::create(accel_key, accel_mods, hardware_keycode);
+		RSendKey send_key = SendKey::create(accel_key, accel_mods);
 		Glib::ustring str = send_key->get_label();
 		if (row[cols.arg] == str)
 			return;
@@ -982,13 +994,7 @@ void Actions::on_arg_editing_started(Gtk::CellEditable* editable, const Glib::us
 }
 
 const Glib::ustring SendKey::get_label() const {
-	Glib::ustring str = Gtk::AccelGroup::get_label(key, mods);
-	if (key == 0) {
-		char buf[10];
-		snprintf(buf, 9, "0x%x", code);
-		str += buf;
-	}
-	return str;
+	return Gtk::AccelGroup::get_label(key, mods);
 }
 
 const Glib::ustring ModAction::get_label() const {
