@@ -114,8 +114,6 @@ public:
 	PrefSource<ButtonInfo> button;
 	PrefSource<TraceType> trace;
 	PrefSource<bool> advanced_ignore;
-	PrefSource<bool> ignore_grab;
-	PrefSource<bool> timing_workaround;
 	PrefSource<bool> pressure_abort;
 	PrefSource<int> pressure_threshold;
 	PrefSource<bool> proximity;
@@ -135,12 +133,13 @@ public:
 	PrefSource<double> scroll_speed;
 	PrefSource<bool> tray_feedback;
 	PrefSource<bool> show_osd;
+	PrefSource<bool> move_back;
 
 	void init();
 	virtual void timeout();
 };
 
-BOOST_CLASS_VERSION(PrefDB, 15)
+BOOST_CLASS_VERSION(PrefDB, 16)
 
 extern PrefDB prefs;
 
