@@ -22,10 +22,6 @@
 #include <set>
 #include <iostream>
 
-Source<bool> xinput_v(false);
-Source<bool> supports_pressure(false);
-Source<bool> supports_proximity(false);
-
 class Check : private Base {
 	IO<bool> &io;
 	Gtk::CheckButton *check;
@@ -151,7 +147,6 @@ public:
 	}
 };
 
-static bool and_(bool x, bool y) { return x && y; }
 static bool is_custom(TimeoutType profile) { return profile == TimeoutCustom; }
 static bool draw_trace(TraceType t) { return t == TraceDefault || t == TraceShape; }
 
@@ -200,7 +195,7 @@ public:
 		dir += "/.config/autostart";
 		filename = dir + "/easystroke.desktop";
 
-		if (!is_dir(dir)) {
+		if (!is_dir(dir) && mkdir(dir.c_str(), 0777)) {
 			autostart_ok.set(false);
 			return;
 		}
@@ -238,8 +233,6 @@ void remove_last_entry(const Glib::ustring & name) {
 
 Prefs::Prefs() {
 	new Check(prefs.advanced_ignore, "check_advanced_ignore");
-	new Check(prefs.ignore_grab, "check_ignore_grab");
-	new Check(prefs.timing_workaround, "check_timing_workaround");
 
 	new Check(prefs.pressure_abort, "check_pressure_abort");
 	new Adjustment<int>(prefs.pressure_threshold, "adjustment_pressure_threshold");
@@ -276,6 +269,8 @@ Prefs::Prefs() {
 	new Check(prefs.scroll_invert, "check_scroll_invert");
 	new Adjustment<double>(prefs.scroll_speed, "adjustment_scroll_speed");
 
+	new Check(prefs.move_back, "check_move_back");
+
 	new Check(prefs.show_osd, "check_osd");
 
 	Gtk::Button *bbutton, *add_exception, *remove_exception, *add_extra, *edit_extra, *remove_extra;
@@ -290,13 +285,11 @@ Prefs::Prefs() {
 	widgets->get_widget("treeview_devices", dtv);
 	widgets->get_widget("treeview_extra", etv);
 
-	new Sensitive(xinput_v, "hbox_timeout_profile");
-	new Sensitive(xinput_v, "frame_advanced1");
-	new Sensitive(xinput_v, "frame_advanced2");
-	new Sensitive(xinput_v, "frame_advanced3");
-	new Sensitive(*fun2(&and_, xinput_v, *fun(&is_custom, prefs.timeout_profile)), "hbox_timeout");
+	new Sensitive(*fun(&is_custom, prefs.timeout_profile), "hbox_timeout");
+	/*
 	new Sensitive(supports_pressure, "hbox_pressure");
 	new Sensitive(supports_proximity, "check_proximity");
+	*/
 	new Sensitive(*fun(&draw_trace, prefs.trace), "button_color");
 	new Sensitive(*fun(&draw_trace, prefs.trace), "spin_trace_width");
 
@@ -345,14 +338,12 @@ Prefs::Prefs() {
 	for (std::map<std::string, RButtonInfo>::const_iterator i = exceptions.begin(); i!=exceptions.end(); i++) {
 		Gtk::TreeModel::Row row = *(tm->append());
 		row[cols.app] = i->first;
-		row[cols.user_app] = i->first == "" ? _("(window manager frame)") : i->first;
+		row[cols.user_app] = app_name_hr(i->first);
 		row[cols.button] = i->second ? i->second->get_button_text() : _("<App disabled>");
 	}
 }
 
 void Prefs::update_device_list() {
-	if (!grabber->xinput)
-		return;
 	ignore_device_toggled = true;
 	dtm->clear();
 	std::set<std::string> names;
@@ -597,11 +588,10 @@ void Prefs::on_add() {
 		is_new = prefs.exceptions.write_ref(a).insert(
 				std::pair<std::string, RButtonInfo>(str, RButtonInfo())).second;
 	}
-	Glib::ustring user_str = str == "" ? _("(window manager frame)") : str;
 	if (is_new) {
 		Gtk::TreeModel::Row row = *(tm->append());
 		row[cols.app] = str;
-		row[cols.user_app] = user_str;
+		row[cols.user_app] = app_name_hr(str);
 		row[cols.button] = _("<App disabled>");
 		Gtk::TreePath path = tm->get_path(row);
 		tv->set_cursor(path);
